package ru.dbotthepony.kstarbound.io

import com.google.common.collect.ImmutableMap
import com.google.gson.JsonSyntaxException
import com.google.gson.TypeAdapter
import com.google.gson.stream.JsonReader
import com.google.gson.stream.JsonWriter
import it.unimi.dsi.fastutil.objects.Object2ObjectArrayMap

class EnumAdapter<T : Enum<T>>(private val enum: Class<T>) : TypeAdapter<T>() {
	private val mapping: ImmutableMap<String, T> = Object2ObjectArrayMap<String, T>().let {
		for (value in enum.enumConstants) {
			it[value.name] = value
			it[value.name.uppercase()] = value
			it[value.name.lowercase()] = value

			val spaced = value.name.replace('_', ' ')
			val stitched = value.name.replace("_", "")

			it[spaced] = value
			it[spaced.uppercase()] = value
			it[spaced.lowercase()] = value

			it[stitched] = value
			it[stitched.uppercase()] = value
			it[stitched.lowercase()] = value
		}

		ImmutableMap.copyOf(it)
	}

	override fun write(out: JsonWriter, value: T) {
		out.value(value.name)
	}

	override fun read(`in`: JsonReader): T {
		val key = `in`.nextString()
		return mapping[key] ?: throw JsonSyntaxException("Unable to match '$key' against ${enum.canonicalName}")
	}
}
